local cql = ClassicQuestLog

local MAX_QUESTS = 35 -- maximum number of non-hidden quests a play can have in the log
local MAX_BUTTON_WIDTH = 140 -- maximum width of red panel buttons that stretch to fit in resizing window
local FOUR_BUTTON_THRESHOLD = 560 -- below this, show at most four buttons (abandon, share, track and sync/close)
local THREE_BUTTON_THRESHOLD = 450 -- below this, show three buttons (abandon, share, track)

function cql.Chrome:Setup()
    self:SetScript("OnSizeChanged",self.OnSizeChanged)
    self:OnSizeChanged(self:GetWidth(),self:GetHeight())
end

function cql.Chrome:Update()
    self.CloseButton:SetText(cql:GetMode()=="Detail" and CLOSE or BACK)

    -- if resize grip is being shown/hidden then adjust close button position
    if self.ResizeGrip:IsShown() and not cql.settings.ShowResizeGrip then
        self.ResizeGrip:Hide()
        self.CloseButton:SetPoint("BOTTOMRIGHT",-4,4)
    elseif not self.ResizeGrip:IsShown() and cql.settings.ShowResizeGrip then
        self.ResizeGrip:Show()
        self.CloseButton:SetPoint("BOTTOMRIGHT",-18,4)
    end

    --  quest count
    self.CountFrame.Text:SetText(format("%s \124cffffffff%d/%d",QUESTS_COLON,cql.Log:GetNumQuests(),MAX_QUESTS))

    -- map button
    self.MapButton:Update()
    -- sync frame
    self.SyncFrame:Update()

    -- panel buttons
    local questID = cql.Log:GetSelectedQuest()
    self.AbandonButton:SetEnabled(C_QuestLog.CanAbandonQuest(questID) and cql:GetMode()=="Detail")
    self.ShareButton:SetEnabled(C_QuestLog.IsPushableQuest(questID) and IsInGroup() and cql:GetMode()=="Detail")
    -- track button
    local isQuestDisabled = C_QuestLog.IsQuestDisabledForSession(questID)
    local isWatched = C_QuestLog.GetQuestWatchType(questID)~=nil
    self.TrackButton:SetText(isWatched and UNTRACK_QUEST_ABBREV or TRACK_QUEST_ABBREV)
    self.TrackButton:SetEnabled(questID~=0 and (isWatched or not isQuestDisabled) and cql:GetMode()=="Detail")
end

-- map button updated separately since it's disabled in main under player regen disabled/enabled
function cql.Chrome.MapButton:Update()
    local questID = cql.Log:GetSelectedQuest()
    local numQuests = cql.Log:GetNumQuests()
    local mapID = GetQuestUiMapID(questID or 0)
    if mapID~=0 and not InCombatLockdown() and numQuests>0 and cql:GetMode()=="Detail" then
        self.disabledReason = nil
        self:Enable()
        self.Text:SetTextColor(1,0.82,0)
    else
        if numQuests==0 then
            self.disabledReason = "noquests"
        elseif mapID==0 then
            self.disabledReason = "nomap"
        elseif InCombatLockdown() then
            self.disabledReason = "combat"
        else
            self.disabledReason = nil
        end
        self:Disable()
        self.Text:SetTextColor(0.5,0.5,0.5)
    end
end

-- updates the Party Sync button/status at top of window
function cql.Chrome.SyncFrame:Update()
    local shouldShow = self:ShouldBeVisible()
    self:SetShown(shouldShow)

    if shouldShow then -- 370 QuestMapFrame.lua
        local command = self:GetSessionCommand()
        self.Text:SetText(command==Enum.QuestSessionCommand.Start and QUEST_SESSION_START_SESSION or QUEST_SESSION_SESSION_ACTIVE)
        local atlasSuffix = command~=Enum.QuestSessionCommand.Start and "Stop" or ""
        self.Button:SetNormalAtlas(format("QuestSharing-QuestLog-Button%s",atlasSuffix))
        self.Button:SetPushedAtlas(format("QuestSharing-QuestLog-ButtonPressed%s",atlasSuffix))
        self.Button:SetDisabledAtlas(format("QuestSharing-QuestLog-Button%s",atlasSuffix))

        if command==Enum.QuestSessionCommand.SessionActiveNoCommand then
            self.Button:Hide()
            self.Icon:Show()
        else
            self.Button:SetEnabled(cql:GetMode()=="Detail")
            self.Button:Show()
            self.Icon:Hide()
        end

    end
end

function cql.Chrome.SyncFrame.Button:OnClick()
    self:Disable() -- disable the button for 3 seconds (and any event to update will enable it too)
    cql.timer:Start(3,function()
        self:Enable()
    end)
    local command = self:GetParent():GetSessionCommand()
    if command==Enum.QuestSessionCommand.Start then
        QuestSessionManager:StartSession()
    elseif command==Enum.QuestSessionCommand.Stop then
        QuestSessionManager:StopSession()
    end
end

-- replacement for QuestSessionManagerMixin:Get(System)SessionCommand()
function cql.Chrome.SyncFrame:GetSessionCommand()
    local command = C_QuestSession.GetPendingCommand()
    if command == Enum.QuestSessionCommand.None then
        return C_QuestSession.GetAvailableSessionCommand()
    end
    return command
end

-- returns true if in a group that's syncing or can sync
function cql.Chrome.SyncFrame:ShouldBeVisible()
    return C_QuestSession.Exists() or self:GetSessionCommand()~=Enum.QuestSessionCommand.None
end

-- when window changes size, adjust chrome stuff to fit (don't call update in here, because
-- update may call it for resize grip being shown/hidden)
function cql.Chrome:OnSizeChanged(width,height)

    -- resize red panel buttons
    local numButtons = 4
    local showClose = true
    if width < THREE_BUTTON_THRESHOLD then
        showClose = false
        numButtons = numButtons - 1
    end
    local buttonWidth = min(MAX_BUTTON_WIDTH,floor(((width-14-(cql.settings.ShowResizeGrip and 12 or 0))/numButtons)+0.5))
    for _,button in ipairs(self.PanelButtons) do
        button:SetWidth(buttonWidth)
        if button.parentKey=="CloseButton" then
            button:SetShown(showClose)
        end
    end

    -- resize sync status at top of window
    if self.SyncFrame:ShouldBeVisible() then
        local syncWidth = ceil(self.SyncFrame.Text:GetStringWidth() + self.SyncFrame.Button:GetWidth())
        if width-syncWidth>400 then -- if room for text, display icon+text
            self.SyncFrame.Icon:ClearAllPoints()
            self.SyncFrame.Icon:SetPoint("RIGHT",self.SyncFrame.Text,"LEFT",-4,0)
            self.SyncFrame.Button:ClearAllPoints()
            self.SyncFrame.Button:SetPoint("RIGHT",self.SyncFrame.Text,"LEFT",-4,0)
            self.SyncFrame.Text:Show()
            self.MapButton.Text:Show()
            self.SyncFrame:SetPoint("TOP",44,-27)
        elseif width-32>400 then -- if room for icon only, display icon without text
            self.SyncFrame.Icon:ClearAllPoints()
            self.SyncFrame.Icon:SetPoint("CENTER")
            self.SyncFrame.Button:ClearAllPoints()
            self.SyncFrame.Button:SetPoint("CENTER")
            self.SyncFrame.Text:Hide()
            self.MapButton.Text:Show()
            self.SyncFrame:SetPoint("TOP",44,-27)
        else -- party sync is active but no room for indicator (Stop Party Sync on panel button is sufficient)
            self.MapButton.Text:Hide()
            self.SyncFrame:SetPoint("TOP",80,-27)
        end
    end

end

function cql.Chrome.ResizeGrip:OnMouseDown()
    cql:StartSizing()
end

function cql.Chrome.ResizeGrip:OnMouseUp()
    cql:StopMovingOrSizing()
    cql:UpdateSavedPosition()
end

function cql.Chrome.CloseButton:OnClick()
    if cql:GetMode()=="Detail" then
        cql:Toggle()
    else
        cql:SetMode("Detail")
    end
end

function cql.Chrome.AbandonButton:OnClick()
    local questID = cql.Log:GetSelectedQuest()
    if questID and questID~=0 then
        C_QuestLog.SetAbandonQuest()
        local itemIDs = C_QuestLog.GetAbandonQuestItems()
        local itemNames = {}
        local item = Item:CreateFromItemID(0)
        if itemIDs then
            for _,itemID in ipairs(itemIDs) do
                item:SetItemID(itemID)
                local itemName = item:GetItemName()
                if itemName then
                    tinsert(itemNames,itemName)
                end
            end
        end
        local title = C_TaskQuest.GetQuestInfoByQuestID(questID) or C_QuestLog.GetTitleForQuestID(questID) or ""
        local confirmText = #itemNames>0 and format(ABANDON_QUEST_CONFIRM_WITH_ITEMS,title,table.concat(itemNames,", ")) or format(ABANDON_QUEST_CONFIRM,title)
        cql.Dialog:Confirm(confirmText,function()
            local questID = C_QuestLog.GetAbandonQuest()
            if questID and questID~=0 then
                local nextQuestID,nextHeader = cql.Log:GetNextQuestID(questID)
                C_QuestLog.AbandonQuest()
                PlaySound(SOUNDKIT.IG_QUEST_LOG_ABANDON_QUEST)
                if nextQuestID then
                    C_QuestLog.SetSelectedQuest(nextQuestID)
                    if nextHeader then
                        cql.settings.CollapsedHeaders[nextHeader] = nil
                    end
                end
            end
        end)
    end
end

function cql.Chrome.ShareButton:OnClick()
    local questID = cql.Log:GetSelectedQuest()
    if questID and questID~=0 and C_QuestLog.IsPushableQuest(questID) then
        QuestLogPushQuest()
        PlaySound(SOUNDKIT.IG_QUEST_LOG_OPEN)
    end
end

function cql.Chrome.TrackButton:OnClick()
    cql.Log:ToggleQuestWatch()
    cql.Dialog:Hide()
end

function cql.Chrome.OptionsButton:OnClick()
    cql:SetMode(cql:GetMode()~="Options" and "Options" or "Detail")
end

function cql.Chrome.MapButton:OnClick()
    local questID = cql.Log:GetSelectedQuest()
    if not WorldMapFrame:IsVisible() then
        ToggleWorldMap()
    end
    if questID and questID~=0 then
        local currentMapID = C_Map.GetBestMapForUnit("player")
        local questMapID = GetQuestUiMapID(questID,true)
        -- if on a different map than the quest, go to the map for the quest rather than a waypoint
        if questMapID and questMapID~=0 then
            WorldMapFrame:SetMapID(questMapID)
        end
        WorldMapFrame:PingQuestID(questID)
    end
    cql.Dialog:Hide()
end

function cql.Chrome.MapButton:OnEnter()
    if self.disabledReason then
        GameTooltip:ClearAllPoints()
        GameTooltip:SetPoint("TOPRIGHT", self, "BOTTOMRIGHT", 0, 0)
        GameTooltip:SetOwner(self, "ANCHOR_PRESERVE")
        if self.disabledReason == "noquests" then
            GameTooltip:SetText("No quests")
            GameTooltip:AddLine("Try again after you've picked up a quest.",0.75,0.75,0.75)
        elseif self.disabledReason == "combat" then
            GameTooltip:SetText(ERR_AFFECTING_COMBAT)
            GameTooltip:AddLine("Try again when out of combat.",0.75,0.75,0.75)
        elseif self.disabledReason == "nomap" then
            GameTooltip:SetText("No map for this quest")
            GameTooltip:AddLine("Blizzard has not defined a map for this quest.",0.75,0.75,0.75)
        else
            return
        end
        GameTooltip:Show()
    end
end

function cql.Chrome.MapButton:OnLeave()
    GameTooltip:Hide()
end
